package org.beliaj.knots.diagram.impl;

import java.util.*;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.beliaj.knots.diagram.IIntersection;
import org.beliaj.knots.diagram.IKnotDiagram;
import org.beliaj.knots.diagram.ILine;
import org.beliaj.knots.diagram.IPoint;

/**
 * ,     
 *      .
 * 
 * @see IKnotDiagram
 * @see IPoint
 * @see ILine
 * @see IIntersection
 * 
 * @author  
 * @version 1.1
 */
public class KnotDiagram implements IKnotDiagram{
	/**
	 *  
	 */
	private int myPointCount = 0;
	/**
	 *  
	 */
	private int myLineCount = 0;
	/**
	 *  
	 */
	private int myIntersectionCount = 0;
	
	/**
	 *  
	 */
	private LinkedList<IPoint> myPointsList = new LinkedList<IPoint>();
	
	/**
	 *  
	 */
	private LinkedList<ILine> myLinesList = new LinkedList<ILine>();
	
	/**
	 *     
	 */
	private List<ChangeListener> myChangeListeners = new LinkedList<ChangeListener>();
	/**
	 *  ,      
	 */
	private int mySinglePoints;
	
	/**
	 *  .
	 * @param x x-
	 * @param y y-
	 */
	public IPoint addPoint(int x, int y){
		Point p = new Point(x,y, this.myPointCount);
		
		this.myPointCount++;
		this.mySinglePoints++;
		
		this.myPointsList.add(p);
		
		this.sendChangeEvent(DiagramChangesEnum.ADD_POINT);
		return p;
	}
	
	/**
	 *  .
	 * @param ppoint  
	 * @return   
	 */
	public boolean removePoint(IPoint ppoint){
		if (ppoint == null){
			return false;
		}
		
		//System.out.println("before "+this.mySinglePoints);
		
		Point point;
		point = (Point)ppoint;
		//System.out.println("Removing " + point + " " + point.myLines);
		boolean b = this.myPointsList.remove(point);
		if (point.getLines().size() > 0){
			this.removeLine(point.getLines().get(0));
			//System.out.println("Removing 2 " + point + " " + point.myLines);
			if (point.getLines().size() > 0){
				this.removeLine(point.getLines().get(0));
				//System.out.println("Removing 3 " + point + " " + point.myLines);
			}
			point.getLines().clear();
		}
		
		//System.out.println("in0 "+this.mySinglePoints);
		
		if (b){
			//System.out.println("in "+this.mySinglePoints);
			this.mySinglePoints--;
		}
		
		this.sendChangeEvent(DiagramChangesEnum.DELETE);
		//System.out.println("after "+this.mySinglePoints);
		return b;
	}
	
	/**
	 *   .
	 * @return  
	 */
	public List<IPoint> getPoints(){
		return Collections.unmodifiableList(this.myPointsList);
	}
	
	/**
	 *   .
	 * @return  
	 */
	public List<ILine> getLines(){
		return Collections.unmodifiableList(this.myLinesList);
	}
	
	/**
	 *   .
	 * @param pp1  
	 * @param pp2  
	 */
	public ILine connectPoints(IPoint pp1, IPoint pp2){
		if (!this.myPointsList.contains(pp1)){
			throw new MixedDiagramsException("Spesified point ( " + pp1 +" ) do not belong to the diagram");
		}
		if (!this.myPointsList.contains(pp2)){
			throw new MixedDiagramsException("Spesified point ( " + pp2 +" ) do not belong to the diagram");
		}
		Point p1 = (Point)pp1;
		Point p2 = (Point)pp2;
		
		if (p1.getLines().size() == 1){
			this.mySinglePoints--;
		}
		
		if (p2.getLines().size() == 1){
			this.mySinglePoints--;
		}
		
		Line line;
		if (p1.getLine().size() == 1){
			if (p1.getLine().get(0).getSPoint().equals(p1)){
				line = new Line(p2, p1, this.myLineCount);
			} else {
				line = new Line(p1, p2, this.myLineCount);
			}
		} else if (pp2.getLine().size() == 1){
			if (p2.getLine().get(0).getFPoint().equals(p2)){
				line = new Line(p2, p1, this.myLineCount);
			} else {
				line = new Line(p1, p2, this.myLineCount);
			}
		} else {
			line = new Line(p1, p2, this.myLineCount);
		}
		
		this.myLineCount++;
		this.myLinesList.add(line);
		this.updateOtherLines(line);
		//    
		
		ListIterator<ILine> it = this.myLinesList.listIterator();
		while (it.hasNext()){
			ILine tmpLine = it.next();
			if (Intersection.checkLinesForIntersection(tmpLine, line) && !(line.equals(tmpLine))){
				new Intersection(tmpLine, line, this.myIntersectionCount);
				this.myIntersectionCount++;
			}
		}
		
		this.sendChangeEvent(DiagramChangesEnum.ADD_LINE);
		return line;
	}
	
	/**
	 *  .
	 * @param lline  
	 * @return   
	 */
	public boolean removeLine(ILine lline){
		if (lline == null){
			return false;
		}
		Line line;
		line = (Line)lline;
		boolean b = this.myLinesList.remove(line);
		int count1 = line.getFPoint().getLine().size();
		int count2 = line.getSPoint().getLine().size();
		((Point)line.getFPoint()).removeLine(line);
		((Point)line.getSPoint()).removeLine(line);
		
		
		//       
		ListIterator<IIntersection> it = line.getIntersectionsList().listIterator();
			
		while (it.hasNext()){
				
			Intersection intrs = (Intersection)it.next();
			if (intrs.getUpLine().equals(line)){
				((Line)intrs.getDownLine()).getIntersectionsList().remove(intrs);
			} else {
				((Line)intrs.getUpLine()).getIntersectionsList().remove(intrs);
			}
			
		}
	
		if (b){
			if ((count1 == 2)){ 
				this.mySinglePoints += 1;
			}
			if ((count2 == 2)){ 
				this.mySinglePoints += 1;
			}
		}
		
		this.sendChangeEvent(DiagramChangesEnum.DELETE);
		return b;
	}
	
	public String toString(){
		StringBuffer buffer = new StringBuffer();
		StringBuffer buffer2 = new StringBuffer();
		buffer2.append("Intersections:\n");
		buffer.append("Points:\n");
		for (IPoint point : this.myPointsList) {
		  buffer.append(point);
		  buffer.append('\n');
		}
		
		buffer.append("Lines:\n");
		ListIterator<ILine> it2 = this.myLinesList.listIterator();
		
		while (it2.hasNext()){
			ILine line = it2.next();
			buffer.append(line);
			buffer.append('\n');
			
			ListIterator<IIntersection> it3 = line.getIntersections().listIterator();
			while (it3.hasNext()){
				IIntersection ii = it3.next();
				if (line.equals(ii.getUpLine())){
					buffer2.append(ii+"\n");
				}
			}
		}
		
		return buffer.toString() +  buffer2.toString();
	}

	/**
	 * 
	 */
	public IPoint findPoint(int x, int y, int epsilon) {
		
		IPoint pp = null;
		int cr2 = Integer.MAX_VALUE;
		for (IPoint p : this.myPointsList){
			int r2 = (p.getX() - x)*(p.getX() - x) + (p.getY() - y)*(p.getY() - y);
			if ((r2 <= epsilon*epsilon) && (r2 < cr2)){
				pp = p;
				cr2 = r2;
			}
		}
		
		return pp;
	}

	/**
	 * 
	 */
	public ILine findLine(int x, int y, int epsilon) {
		
		ILine resultLine = null;
		double cr2 = Double.MAX_VALUE;
		for (ILine line : this.myLinesList){
			int xF = line.getFPoint().getX();
			int xS = line.getSPoint().getX();
			int yF = line.getFPoint().getY();
			int yS = line.getSPoint().getY();
			int ab2 = (xF - x)*(xF - x) + (yF - y)*(yF - y);
			int bc2 = (xS - x)*(xS - x) + (yS - y)*(yS - y);
			//    ( )
			int ac2 = (xS - xF)*(xS - xF) + (yS - yF)*(yS - yF);
			//    
			if ((ac2 + bc2 - ab2 < 0) || (ac2 + ab2 - bc2 < 0)){
				//System.out.println("bad \n" + (ac2 + bc2 - ab2) + " " + (ac2 + ab2 - bc2) + "\n (x y) " +
				//		x + " " + y + " \n (F) " + xF + " " + yF + " \n (S) " + xS + " " + yS + "\n ab2 " + 
				//		ab2 + " bc2 " + bc2 + " ac2 " + ac2);
				continue;
			}
			
			// ,   
			double h2 = ab2 - (ab2 + ac2 - bc2)*((ab2 + ac2 - bc2)/(4*(double)ac2));
			
			//System.out.println(h2 + "\n ab2 " + 
			//		ab2 + " bc2 " + bc2 + " ac2 " + ac2);
			
			if ((h2 <= epsilon*epsilon) && (h2 < cr2)){
				
				resultLine = line;
				cr2 = h2;
			}
		}
		
		return resultLine;
	}

	/**
	 * 
	 */
	public void resize(double coeff) {
		for (IPoint p : this.myPointsList){
			Point pp = (Point)p;
			pp.setX((int)Math.round(p.getX()*coeff));
			pp.setY((int)Math.round(p.getY()*coeff));
		}
	}

	/**
	 * 
	 */
	public void setIntersectionType(ILine line1, ILine line2) {
		for (IIntersection ii: line1.getIntersections()){
			if (line1.equals(ii.getDownLine()) && line2.equals(ii.getUpLine())){
				((Intersection)ii).swapLines();
				this.sendChangeEvent(DiagramChangesEnum.SWAP);
			}
		}
	}

	/**
	 * 
	 */
	public void addChangeListener(ChangeListener dcl) {
		this.myChangeListeners.add(dcl);
		
	}
	
	/**
	 * 
	 * @param obj
	 */
	private void sendChangeEvent(Object obj){
		ChangeEvent e = new ChangeEvent(obj);
		for (ChangeListener chl: this.myChangeListeners){
			chl.stateChanged(e);
		}
	}
	
	/**
	 * 
	 */
	public void movePoint(IPoint ipoint, int i, int j) {
		if (!this.myPointsList.contains(ipoint)){
			throw new MixedDiagramsException("Spesified point " + ipoint +" do not belong to the diagram");
		}
		Point point = (Point)ipoint;
		point.move(i, j);
		
		boolean b = false;
		
		for (ILine line: point.getLines()){
			
			HashMap<ILine, Boolean> deleted = new HashMap<ILine, Boolean>();
			HashSet<ILine> born = new HashSet<ILine>();
			
			ListIterator<ILine> it = this.myLinesList.listIterator();
			while (it.hasNext()){
				ILine tmpLine = it.next();
				//System.out.println("tmpLine: "+tmpLine);
				IIntersection ii = this.getIntr(line,tmpLine);
				if (Intersection.checkLinesForIntersection(tmpLine, line) && !(line.equals(tmpLine))){
					//System.out.println("if");
					if (ii == null){
						//System.out.println("ii "+ii);
						b = true; 
						new Intersection(tmpLine, line, this.myIntersectionCount);
						born.add(tmpLine);
						this.myIntersectionCount++;
					}
				} else {
					//System.out.println("else");
					if ( ii != null){
						b = true; 
						//System.out.println("??");
						((Line)line).getIntersectionsList().remove(ii);
						((Line)tmpLine).getIntersectionsList().remove(ii);
						deleted.put(tmpLine, ii.getUpLine().equals(line));
					}
				}
			}
			
			for (ILine line1 : deleted.keySet()){
				ILine next = this.nextLine(line1);
				ILine prev = this.previousLine(line1);
				if (born.contains(next)){
					if (deleted.get(line1)){
						this.setIntersectionType(line, next);
					} else {
						this.setIntersectionType(next, line);
					}
					
				}
				
				if (born.contains(prev)){
					if (deleted.get(line1)){
						this.setIntersectionType(line, prev);
					} else {
						this.setIntersectionType(prev, line);
					}
					
				}
			}
		}
		if (!b){
			this.sendChangeEvent(DiagramChangesEnum.MOVE);
		} else {
			this.sendChangeEvent(DiagramChangesEnum.MOVE_INTERSECTION);
		}
	}
	
	private ILine nextLine(ILine line){
		ILine line1;
		if (line.getSPoint().getLine().size() == 1){
			return null;
		}
		if (line.getSPoint().getLine().get(0).equals(line)){
			line1 = line.getSPoint().getLine().get(1);
		} else {
			line1 = line.getSPoint().getLine().get(0);
		}
		
		return line1;
	}
	
	private ILine previousLine(ILine line){
		ILine line1;
		if (line.getFPoint().getLine().size() == 1){
			return null;
		}
		if (line.getFPoint().getLine().get(0).equals(line)){
			line1 = line.getFPoint().getLine().get(1);
		} else {
			line1 = line.getFPoint().getLine().get(0);
		}
		
		return line1;
	}
	
	/**
	 * 
	 * @param line1
	 * @param line2
	 * @return
	 */
	private IIntersection getIntr(ILine line1, ILine line2){
		if (line1.equals(line2)){
			return null;
		}
		for (IIntersection intr: line1.getIntersections()){
			if (intr.getDownLine().equals(line2) || intr.getUpLine().equals(line2)){
				return intr;
			}
		}
		return null;
	}

	/**
	 * 
	 */
	public IIntersection findIntersection(int x, int y, int epsilon) {
		ILine line1 = this.findLine(x, y, epsilon);
		IIntersection ci = null;
		double cr2 = Double.MAX_VALUE;
		if (line1 != null){
			for (IIntersection intr: line1.getIntersections()){
				double dx = intr.getPosition()[0] - x;
				double dy = intr.getPosition()[1] - y;
			
				double r2 = dx*dx + dy*dy;
				if ((r2 < epsilon*epsilon) && (r2 < cr2)){
					ci = intr;
					cr2 = r2;
				}
			}
		}
		return ci;
	}

	/**
	 * 
	 */
	public int getMinXCoordinate() {
		int currentMin = Integer.MAX_VALUE;
		
		for (IPoint p: this.myPointsList){
			if (p.getX() < currentMin){
				currentMin = p.getX();
			}
		}
		
		return currentMin;
	}

	/**
	 * 
	 */
	public int getMaxYCoordinate() {
		int currentMax = Integer.MIN_VALUE;
		
		for (IPoint p: this.myPointsList){
			if (p.getY() > currentMax){
				currentMax = p.getY();
			}
		}
		
		return currentMax;
	}

	/**
	 * 
	 */
	public int getMinYCoordinate() {
		int currentMin = Integer.MAX_VALUE;
		
		for (IPoint p: this.myPointsList){
			if (p.getY() < currentMin){
				currentMin = p.getY();
			}
		}
		
		return currentMin;
	}

	/**
	 * 
	 */
	public int getMaxXCoordinate() {
		int currentMax = Integer.MIN_VALUE;
		
		for (IPoint p: this.myPointsList){
			if (p.getX() > currentMax){
				currentMax = p.getX();
			}
		}
		
		return currentMax;
	}

	/**
	 * 
	 */
	public boolean swapLines(IIntersection intersection) {
		if (intersection == null){
			return false;
		}
		if (!this.myLinesList.contains(intersection.getDownLine()) ||
				!this.myLinesList.contains(intersection.getUpLine())){
			throw new MixedDiagramsException("Spesified point " + intersection +" do not belong to the diagram");
		}
		
		((Intersection)intersection).swapLines();
		this.sendChangeEvent(DiagramChangesEnum.SWAP);
		return true;
	}
	
	/**
	 * 
	 * @param line
	 */
	private void updateOtherLines(Line line){
		IPoint fp = line.getFPoint();
		IPoint sp = line.getSPoint();
		Line line1 = null;
		
		//System.out.println("I: "+this.myI);
		//System.out.println("line: "+line);
		//System.out.println("fp: "+fp);
		//System.out.println("sp: "+sp);
		
		if (fp.getLine().size() > 1){
			if ((line == fp.getLine().get(0))){
				line1 = (Line)fp.getLine().get(1);
			} else {
				line1 = (Line)fp.getLine().get(0);
			}
			
			//System.out.println("1 line1: "+line1);
			
			if (line1.getFPoint().equals(fp)){
				line1.changeDirection();
				//System.out.println("1 ch line1: "+line1);
				this.updateOtherLines(line1);
			}
		}
		
		if (sp.getLine().size() > 1){
			if ((line == sp.getLine().get(0))){
				line1 = (Line)sp.getLine().get(1);
			} else {
				line1 = (Line)sp.getLine().get(0);
			}
			
			//System.out.println("2 line1: "+line1);
			
			if (line1.getSPoint().equals(sp)){
				line1.changeDirection();
				//System.out.println("2 ch line1: "+line1);
				this.updateOtherLines(line1);
			}
		}
		
	}

	/**
	 * 
	 */
	public void changeLineDirection(ILine line) {
		if (!this.myLinesList.contains(line)){
			throw new MixedDiagramsException("Spesified line " + line +" do not belong to the diagram");
		}
		
		line.changeDirection();
		
		this.updateOtherLines((Line)line);
		
		this.sendChangeEvent(DiagramChangesEnum.SHANGE_DIRECTION);
	}

	/**
	 * 
	 */
	public boolean isComplete() {
		return ((this.mySinglePoints == 0) && (this.getLines().size() > 0));
	}
}
