package org.beliaj.knots.main;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.beans.PropertyVetoException;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.beliaj.knots.diagram.*;
import org.beliaj.knots.diagram.impl.KnotDiagram;
import org.beliaj.knots.io.impl.BasicDiagramReader;
import org.beliaj.knots.io.impl.BasicDiagramWriter;
import org.beliaj.knots.io.impl.BasicFileFilter;
import org.beliaj.knots.painters.*;
import org.beliaj.knots.painters.impl.ActionEnum;
import org.beliaj.knots.painters.impl.PaintDiagramClient;
import org.xml.sax.SAXException;

/**
 *   .
 * 
 * @author Julia
 *
 */
public class Main {
	
	/**
	 *   .
	 */
	static int openFrameCount = 0;
	
	/**
	 * .
	 */
	int myInset = 50;
	
	/**
	 *  .
	 */
	private Dimension myScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
	
	/**
	 *   .
	 */
	private JFrame myFrame;
	private JToolBar myPanel;
	private JMenuBar myMenuBar;
	private JMenu myWindowMenu;
	private JMenu mySettingsMenu;
	private ButtonGroup myWindowGroup = new ButtonGroup();
	
	ArrayList<InternalDiagramFrame> myInternalFrames = new ArrayList<InternalDiagramFrame>();
	InternalDiagramFrame myCurrentDiagramFrame;
	private List<ButtonAction> myDiagramActions = new ArrayList<ButtonAction>();
	private List<Action> myFileActions = new ArrayList<Action>();
	private List<Action> myMathActions = new ArrayList<Action>();
	private List<SimpleButtonAction> mySettingsActions = new ArrayList<SimpleButtonAction>();
	
	private String[] myIcons = new String[6];
	
	{
		this.myIcons[0] = "/org/beliaj/knots/main/img/point.png";
		this.myIcons[1] = "/org/beliaj/knots/main/img/line.png";
		this.myIcons[2] = "/org/beliaj/knots/main/img/move.png";
		this.myIcons[3] = "/org/beliaj/knots/main/img/del.png";
		this.myIcons[4] = "/org/beliaj/knots/main/img/intr.png";
		this.myIcons[5] = "/org/beliaj/knots/main/img/dir4.png";
		
	}
	
	protected int myUntitledCount;

	private JDesktopPane myDesktop;
	
	/**
	 *  GUI.
	 */
	private static void createAndShowGUI() {
		new Main();
	}
	
	/**
	 * .
	 *
	 */
	private Main(){
		JFrame.setDefaultLookAndFeelDecorated(true);

	    // 
	    this.myFrame = new JFrame("The Knots");
	    
	    this.myFrame.setBounds(myInset, myInset, myScreenSize.width  - myInset*2, myScreenSize.height - myInset*2);
	    
	    myDesktop = new JDesktopPane();
	    
	    this.createFileActions();
	    this.createDiagramActions();
	    this.createMathActions();
	    this.createSettingsActions();
	    this.createMenuBar();
	    this.createToolBar();
        
	    
        this.disableAll(true);
        this.restoreSettingsMenu();
        //test();
        
        WindowAdapter wa = new WindowAdapter(){
        	public void windowClosing(WindowEvent e){
        		Main.this.myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        		for (InternalDiagramFrame iframe: Main.this.myInternalFrames){
        			iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    				if (iframe.hasChanges()){
    					int i = JOptionPane.showConfirmDialog(iframe, "Save " + iframe.getTitle() + " before closing?", "",JOptionPane.YES_NO_CANCEL_OPTION);
    					if (i == JOptionPane.YES_OPTION){
    						if (Main.this.saveDoc()){
    							iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    						} else {
    							iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    							Main.this.myFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    						}
    					} else if(i == JOptionPane.NO_OPTION){
    						iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    					} else {
    						iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    						Main.this.myFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    						break;
    						//Main.this.close = false;
    					}
    				}
        		}
        	}
        };
        
        this.myFrame.addWindowListener(wa);
        
        this.myFrame.setJMenuBar(this.myMenuBar);
        //this.myFrame.setLayout(new FlowLayout());
        this.myFrame.setContentPane(this.myDesktop);
        
        this.myFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        
        this.myFrame.setVisible(true);
	}
	
	/**
	 *    action,    ,  
	 *
	 */
	private class ButtonAction extends AbstractAction{
		
		private LinkedList<AbstractButton> myButtons = new LinkedList<AbstractButton>();
		private ActionEnum myActionCode;
		
		public ButtonAction(String string, ActionEnum actionCode) {
			super(string);
			myActionCode = actionCode;
		}

		public void addButton(AbstractButton ab){
			this.myButtons.add(ab);
		}

		public void actionPerformed(ActionEvent e) {
			for (AbstractButton ab: this.myButtons){
				ab.setSelected(true);
			}
			//Main.this.myCurrentAction = myActionCode;
			//for (InternalDiagramFrame iframe: Main.this.myInternalFrames){
			//	iframe.myPainter.setCurrentAction(Main.this.myCurrentAction);
			//	iframe.reloadAllListeners();
			//}
			Main.this.myCurrentDiagramFrame.myPainter.setCurrentAction(myActionCode);
			Main.this.myCurrentDiagramFrame.reloadAllListeners();
		}
	}
	
	private class SimpleButtonAction extends AbstractAction{
		
		private LinkedList<AbstractButton> myButtons = new LinkedList<AbstractButton>();
		
		public SimpleButtonAction(String string) {
			super(string);
		}

		public void addButton(AbstractButton ab){
			this.myButtons.add(ab);
		}

		public void actionPerformed(ActionEvent e) {
			//for (AbstractButton ab: this.myButtons){
			//	ab.setSelected(true);
			//}
		}
		
		public boolean isActionPossible(){
			return true;
		}

		public boolean isSet() {
			return false;
		}
	}
	
	/**
	 *  action-  .
	 *
	 */
	private void createDiagramActions(){
		ButtonAction action = new ButtonAction("Add Point", ActionEnum.ADD_POINT);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_P, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
		
		
		action = new ButtonAction("Add Line", ActionEnum.ADD_LINE);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_L, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
		
		action = new ButtonAction("Move", ActionEnum.MOVE);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
		
		action = new ButtonAction("Delete", ActionEnum.DELETE);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_X, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
		
		action = new ButtonAction("Swap", ActionEnum.SWAP);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
		
		action = new ButtonAction("Direction", ActionEnum.SHANGE_DIRECTION);
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_D, InputEvent.CTRL_MASK));
		myDiagramActions.add(action);
	}
	
	private void createSettingsActions(){
		this.mySettingsActions.add( new SimpleButtonAction("Show directions"){
			public void actionPerformed(ActionEvent e) {
				super.actionPerformed(e);
				Main.this.myCurrentDiagramFrame.myPainter.viewDirections();
				Main.this.updateDiagramActions();
				Main.this.myCurrentDiagramFrame.repaint();
			}
			
			public boolean isActionPossible(){
				if (Main.this.myCurrentDiagramFrame != null){
					//System.out.println("is action possible"+(Main.this.myCurrentDiagramFrame.myPainter.getKnotDiagram().getLines().size() > 0));
					return (Main.this.myCurrentDiagramFrame.myPainter.getKnotDiagram().getLines().size() > 0);
				} else return false;
			}
			
			public boolean isSet(){
				if (Main.this.myCurrentDiagramFrame != null){
					return (Main.this.myCurrentDiagramFrame.myPainter.hasDirections());
				} else return false; 
			}
		});
	}
	
	private void createMathActions(){
		this.myMathActions.add( new AbstractAction("Count Jones Polynom"){
			public void actionPerformed(ActionEvent e) {
				//if (Main.this.myCurrentDiagramFrame != null){
					Main.this.myCurrentDiagramFrame.viewJonesPolynom();
				//}
			}
		});
		this.myMathActions.get(0).putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_J, InputEvent.CTRL_MASK));
	}
	
	/**
	 * 
	 *   action -    .
	 */
	private void createFileActions(){
		Action action = new AbstractAction("New"){
			public void actionPerformed(ActionEvent e) {
				Main.this.newDoc();
			}
		};
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		myFileActions.add(action);
		
		action = new AbstractAction("Open"){
			
			//      
			private String myCurrentFolder = System.getProperty("user.dir");
			
			public void actionPerformed(ActionEvent e) {
				String str = Main.this.openDoc(this.myCurrentFolder);
				if (str != null){
					this.myCurrentFolder = str;
				}
			}
		};
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_O, InputEvent.CTRL_MASK));
		myFileActions.add(action);
		
		action = new AbstractAction("Save"){
			public void actionPerformed(ActionEvent e) {
				Main.this.saveDoc();
			}
		};
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		myFileActions.add(action);
		
		action = new AbstractAction("SaveAs"){
			public void actionPerformed(ActionEvent e) {
				Main.this.saveAsDoc();
			}
		};
		myFileActions.add(action);
		
		myFileActions.add(null);
		
		action = new AbstractAction("Exit"){
			public void actionPerformed(ActionEvent e) {
				boolean b = false;
        		for (InternalDiagramFrame iframe: Main.this.myInternalFrames){
    				if (iframe.hasChanges()){
    					int i = JOptionPane.showConfirmDialog(iframe, "Save " + iframe.getTitle() + " before closing?", "",JOptionPane.YES_NO_CANCEL_OPTION);
    					if (i == JOptionPane.YES_OPTION){
    						if (Main.this.saveDoc()){
    							iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    						} else {
    							b = true;
    							iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    						}
    					} else if(i == JOptionPane.NO_OPTION){
    						iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
    					} else {
    						b = true;
    						iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    						break;
    						//Main.this.close = false;
    					}
    				}
        		}
				
        		if (!b){
        			myFrame.dispose();
        		}
			}
		};
		action.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Q, InputEvent.CTRL_MASK));
		myFileActions.add(action);
	}
	
	private boolean saveDoc() {
		if (this.myCurrentDiagramFrame.myFileName != ""){
			
			BasicDiagramWriter dw = new BasicDiagramWriter(this.myCurrentDiagramFrame.myFileName);
			
			try {
				
				dw.write(this.myCurrentDiagramFrame.getKnotDiaram(),
						this.myCurrentDiagramFrame.getSize());
				this.myCurrentDiagramFrame.setSaved(true);
				
			} catch (IOException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + this.myCurrentDiagramFrame.myFileName
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			} catch (ParserConfigurationException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + this.myCurrentDiagramFrame.myFileName
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			} catch (TransformerException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + this.myCurrentDiagramFrame.myFileName
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			}
			
			return true;
			
		} else {
			return this.saveAsDoc();
		}
		
	}

	private boolean saveAsDoc() {
		JFileChooser chooser = new JFileChooser();
		BasicFileFilter filter = new BasicFileFilter();
	    chooser.setFileFilter(filter);
		chooser.setSelectedFile(new File(this.myCurrentDiagramFrame.myFileName));
		chooser.setAcceptAllFileFilterUsed(false);
		chooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
	    int returnVal = chooser.showSaveDialog(myFrame);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	String str = chooser.getSelectedFile().getPath();
	    	String str1 = chooser.getSelectedFile().getName();
	    	if (!filter.accept(str)){
				str += "."+BasicDiagramReader.getFilter();
				str1 += "."+BasicDiagramReader.getFilter(); 
			}
	    	BasicDiagramWriter dw = new BasicDiagramWriter(str);
	    	this.myCurrentDiagramFrame.myFileName = str;
	    	this.myCurrentDiagramFrame.setTitle(str1);
	    	this.myCurrentDiagramFrame.setSaved(true);
	    	int i = this.myInternalFrames.indexOf(this.myCurrentDiagramFrame);
	    	JCheckBoxMenuItem jcb = (JCheckBoxMenuItem) this.myWindowMenu.getItem(i);
			FocusAction action = (FocusAction)jcb.getAction();
			jcb.setText(this.myInternalFrames.get(i).getTitle());
			action.setNumber(i+1);
	    	//System.out.println(Main.this.myCurrentDiagramFrame.getKnotDiaram());
			try {
				dw.write(this.myCurrentDiagramFrame.getKnotDiaram(),
						this.myCurrentDiagramFrame.getSize());	
			} catch (IOException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + str
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			} catch (ParserConfigurationException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + str
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			} catch (TransformerException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + str
						+" is bad.", "Error writing", JOptionPane.ERROR_MESSAGE);
			}
			return true;
	    } else {
	    	return false;
	    }
	}

	private String openDoc(String currentFolder) {
		//System.out.println("OpenDoc");
		JFileChooser chooser = new JFileChooser();
		FileFilter filter = new BasicFileFilter();
	    chooser.setFileFilter(filter);
		chooser.setCurrentDirectory(new File(currentFolder));
		currentFolder = null;
		chooser.setAcceptAllFileFilterUsed(false);
	    int returnVal = chooser.showOpenDialog(myFrame);
	    if(returnVal == JFileChooser.APPROVE_OPTION) {
	    	BasicDiagramReader dr;
			try {
				dr = new BasicDiagramReader(chooser.getSelectedFile().getPath());
				IKnotDiagram diagram = dr.read();
				
				InternalDiagramFrame iframe = createDiagramFrame(new PaintDiagramClient(diagram, dr.getDimension()), 
						chooser.getSelectedFile().getName(), dr.getDimension());
				
				currentFolder = chooser.getCurrentDirectory().getPath();
				//System.out.println(dr.getDimension());
				iframe.myFileName = chooser.getSelectedFile().getPath();
				iframe.toFront();
			} catch (SAXException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + chooser.getSelectedFile().getPath()
						+" is bad.", "Error", JOptionPane.ERROR_MESSAGE);
			} catch (ParserConfigurationException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + chooser.getSelectedFile().getPath()
						+" is bad.", "Error", JOptionPane.ERROR_MESSAGE);
			} catch (IOException e1) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + chooser.getSelectedFile().getPath()
						+" is bad.", "Error", JOptionPane.ERROR_MESSAGE);
			} catch (RuntimeException re){
				JOptionPane.showMessageDialog(Main.this.myFrame, "File " + chooser.getSelectedFile().getPath()
						+" is bad.", "Error", JOptionPane.ERROR_MESSAGE);
			
			}
			
	    }
	    return currentFolder;
	}

	private void newDoc() {
		IKnotDiagram knotDiagram = new KnotDiagram();
		createDiagramFrame(new PaintDiagramClient(knotDiagram, null),
				"Untitled" + Main.this.myUntitledCount+ ".knd",null);
		this.myUntitledCount++;
	}

	/**
	 *   
	 *
	 */
	private void createToolBar(){
		this.myPanel = new JToolBar(JToolBar.HORIZONTAL);
		
		int i = 0;
		ButtonGroup buttonGroup = new ButtonGroup();
		for (ButtonAction action: Main.this.myDiagramActions){
			
			if (action != null){
				JToggleButton jtb1 = new JToggleButton(action);
				URL u = Main.class.getResource(this.myIcons[i]);
				//System.out.println(u);
				jtb1.setIcon(new ImageIcon(u));
				jtb1.setText("");
				//System.out.println("icon "+ i+ " " + this.myIcons[i]);
				action.addButton(jtb1);
				if (i == 0){
					jtb1.setSelected(true);
				}
				i++;
		
				buttonGroup.add(jtb1);
				this.myPanel.add(jtb1);
			} else {
				this.myPanel.add(new JSeparator());
	            buttonGroup = new ButtonGroup();
			}
			
		}
		//this.myPanel.setOrientation(JToolBar.HORIZONTAL);
		//this.myFrame.getLayeredPane().add(this.myPanel, BorderLayout.PAGE_START);
		this.myPanel.setBounds(0,0,280,50);
		//this.myPanel.setLocation(0,0);
		GridBagLayout lm = new GridBagLayout();
		//lm.
		this.myPanel.setLayout(lm);
		this.myPanel.setFloatable(false);
		//this.myDesktop.add(myPanel);
		this.myDesktop.add(myPanel, BorderLayout.PAGE_START);
		this.myDesktop.setLayer(myPanel, JDesktopPane.PALETTE_LAYER);
		//this.myFrame.getContentPane().add(myPanel);
	}
	
	/**
	 *  
	 *
	 */
	private void createMenuBar(){
		this.myMenuBar = new JMenuBar();
		
		JMenu menu = new JMenu("File");
		for (Action action: this.myFileActions) {
			//System.out.println(action);
		    	
		    if (action != null) {
		    	JMenuItem jrb = new JMenuItem(action);
		        menu.add(jrb);
		    } else {
		        menu.add(new JSeparator());
		    }
		}
		myMenuBar.add(menu);
	
		menu = new JMenu("Diagram");
		ButtonGroup buttonGroup = new ButtonGroup();
		int i = 0;
		menu.add(new JSeparator());
	    for (ButtonAction action: this.myDiagramActions) {
	        if (action != null) {
	        	JCheckBoxMenuItem jrb = new JCheckBoxMenuItem(action);
	            menu.add(jrb);
	            action.addButton(jrb);
	            buttonGroup.add(jrb);
	            if (i == 0){
	            	jrb.setSelected(true);
	            }
	            i++;
	        } else {
	            menu.add(new JSeparator());
	            buttonGroup = new ButtonGroup();
	        }
	    }
	    myMenuBar.add(menu);
	    
	    menu = new JMenu("Math");
	    for (Action action: this.myMathActions) {
	    	
	        if (action != null) {
	        	JMenuItem jrb = new JMenuItem(action);
	            menu.add(jrb);
	            i++;
	        } else {
	    		menu.add(new JSeparator());
	            buttonGroup = new ButtonGroup();
	        }
	    }
	    myMenuBar.add(menu);
	    
	    this.mySettingsMenu = new JMenu("Settings");
	    for (SimpleButtonAction action: this.mySettingsActions) {
	    	if (action != null) {
	        	JCheckBoxMenuItem jrb = new JCheckBoxMenuItem(action);
	        	action.addButton(jrb);
	        	this.mySettingsMenu.add(jrb);
	            i++;
	        } else {
	        	this.mySettingsMenu.add(new JSeparator());
	        }
	    }
	    myMenuBar.add(this.mySettingsMenu);
	   
	    this.myWindowMenu = new JMenu("Window");
	    myMenuBar.add(this.myWindowMenu);
	    
	    menu = new JMenu("Help");
	    Action action = new AbstractAction("About..."){
			public void actionPerformed(ActionEvent e) {
				JOptionPane.showMessageDialog(Main.this.myFrame, "The Knots\n" +
						"version 1.0\n" +
						"written by Julia Beliaeva\n" +
						"2006", "About...", JOptionPane.INFORMATION_MESSAGE);
			}
	    	
	    };
	    JMenuItem jrb = new JMenuItem(action);
        menu.add(jrb);
	    myMenuBar.add(menu);
	  
	}
	
	/**
	 *  action,      .
	 */
	private class FocusAction extends AbstractAction{

		private int myNumber;
		
		public FocusAction(int i, String title) {
			super(title);
			this.myNumber = i-1;
		}

		public void setNumber(int i) {
			this.myNumber = i-1;
		}

		public void actionPerformed(ActionEvent e) {
			try {
				(Main.this.myInternalFrames.get(this.myNumber)).setSelected(true);
			} catch (PropertyVetoException e1) {
				e1.printStackTrace();
			}
		}
		
	}
	
	/**
	 *    .
	 * @param painter
	 * @param dimension 
	 */
    protected InternalDiagramFrame createDiagramFrame(IPainter painter, String title, Dimension dimension) {
    	InternalDiagramFrame iframe = new InternalDiagramFrame(this, painter, title, dimension);
    	
        iframe.setVisible(true);
        this.myCurrentDiagramFrame = iframe;
        this.myCurrentDiagramFrame.myPainter = iframe.myPainter;
        this.myDesktop.add(iframe, JDesktopPane.DEFAULT_LAYER);
        
		iframe.reloadAllListeners();
		JCheckBoxMenuItem jmi = new JCheckBoxMenuItem(new FocusAction(Main.openFrameCount, iframe.getTitle()));
		this.myWindowGroup.add(jmi);
		this.myWindowMenu.add(jmi);
		jmi.setSelected(true);
		
        try {
			iframe.setSelected(true);
		} catch (PropertyVetoException e) {
			e.printStackTrace();
		}
		
		//    
		//   
		InternalFrameListener closeAndAllOtherListener = new InternalFrameAdapter(){
			public void internalFrameClosed(InternalFrameEvent e){
				InternalDiagramFrame iframe = ((InternalDiagramFrame)e.getInternalFrame());
				Main.openFrameCount--;
				if (Main.openFrameCount == 0){
					Main.this.disableAll(true);
				}
				Main.this.myInternalFrames.remove(Main.this.myInternalFrames.indexOf(iframe));
				Main.this.restoreWindowMenu();
	
			}
			
			public void internalFrameClosing(InternalFrameEvent e){
				
				InternalDiagramFrame iframe = ((InternalDiagramFrame)e.getInternalFrame());
				iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
				if (iframe.hasChanges()){
					int i = JOptionPane.showConfirmDialog(iframe, "Save " + iframe.getTitle() + " before closing?", "",JOptionPane.YES_NO_CANCEL_OPTION);
					if (i == JOptionPane.YES_OPTION){
						if (Main.this.saveDoc()){
							iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
						} else {
							iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
						}
					} else if(i == JOptionPane.NO_OPTION){
						iframe.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
					} else {
						iframe.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
					}
				}
			}
			
			public void internalFrameActivated(InternalFrameEvent e){
				Main.this.myCurrentDiagramFrame = (InternalDiagramFrame) e.getInternalFrame();
				Main.this.myWindowMenu.getItem(Main.this.myInternalFrames.indexOf(e.getInternalFrame())).setSelected(true);
				Main.this.updateSettingsActions();
				Main.this.updateDiagramActions();
				Main.this.restoreSettingsMenu();
			}
		};
		
		this.updateSettingsActions();
		iframe.addInternalFrameListener(closeAndAllOtherListener);
		this.updateDiagramActions();
		this.restoreSettingsMenu();
		return iframe;
	}

    /**
     *   Window
     *
     */
	private void restoreWindowMenu() {
		this.myWindowMenu.removeAll();
		JCheckBoxMenuItem jcb;
		int i = 0;
		for (InternalDiagramFrame iframe: this.myInternalFrames){
			FocusAction action = new FocusAction(i+1, iframe.getTitle());
			jcb = new JCheckBoxMenuItem(action);
			this.myWindowMenu.add(jcb);
			this.myWindowGroup.add(jcb);
			if (iframe.isSelected()){
				jcb.setSelected(true);
			}
		}
	}
	
	void restoreSettingsMenu(){
		for (SimpleButtonAction ba: this.mySettingsActions){
			ba.setEnabled(ba.isActionPossible());
		}
	}
	
	void updateSettingsActions(){
		for (SimpleButtonAction ba: this.mySettingsActions){
			for (AbstractButton button: ba.myButtons){
				button.setSelected(ba.isSet());
			}
		}
	}
	
	/**
	 *      
	 * @param arg
	 */
	private void disableAll(boolean arg){
		for (Action action: this.myDiagramActions){
			action.setEnabled(!arg);
		}
		
		for (Action action: this.myMathActions){
			action.setEnabled(!arg);
		}
	}
	
	public void updateDiagramActions(){
		boolean bool = false;
		if (this.myCurrentDiagramFrame.isActionPossible(this.myCurrentDiagramFrame.myPainter.getCurrentAction())){
			bool = true;
		};
		int count = 0;
		for (ButtonAction ba: this.myDiagramActions){
			if ((ba.myActionCode == this.myCurrentDiagramFrame.myPainter.getCurrentAction()) && bool){
				for (AbstractButton b: ba.myButtons){
					b.setSelected(true);
				}
			}
			if (this.myCurrentDiagramFrame.isActionPossible(ba.myActionCode)){
				if (!bool && (count == 0)){
					for (AbstractButton b: ba.myButtons){
						b.setSelected(true);
					}
					count = 1;
					Main.this.myCurrentDiagramFrame.myPainter.setCurrentAction(ba.myActionCode);
					Main.this.myCurrentDiagramFrame.reloadAllListeners();
				}
				ba.setEnabled(true);
			} else {
				ba.setEnabled(false);
			}
		}
		
		if (Main.this.myCurrentDiagramFrame.myPainter.getKnotDiagram().isComplete()){
			for (Action action: this.myMathActions){
				action.setEnabled(true);
			}
		} else {
			for (Action action: this.myMathActions){
				action.setEnabled(false);
			}
		}
		
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
	}
}
