package org.beliaj.knots.painters.impl;

import java.awt.event.MouseEvent;
import java.util.Iterator;

import javax.swing.event.ChangeEvent;

import org.beliaj.knots.diagram.*;
import org.beliaj.knots.painters.IEnum;
import org.beliaj.knots.painters.Actor;

class LineActor extends Actor {
	
	IPoint myCurrentPoint = null; 

	/**
	 * 
	 */
	private final PaintDiagramClient client;

	/**
	 * @param client
	 */
	LineActor(PaintDiagramClient client) {
		this.client = client;
	}

	public void mouseReleased(MouseEvent e) {
		int x = e.getX();
		int y = e.getY();
		
		if (e.getComponent().getHeight() - this.client.myPointDiam < e.getY()){
			y = e.getComponent().getHeight() - this.client.myPointDiam;
		}
		
		if (e.getComponent().getWidth() - this.client.myPointDiam < e.getX()){
			x = e.getComponent().getWidth() - this.client.myPointDiam;
		}
		
		if (e.getY() < this.client.myPointDiam){
			y = this.client.myPointDiam;
		}
		
		if (e.getX() < this.client.myPointDiam){
			x = this.client.myPointDiam;
		}
		
		ILine line = null;
		
		if (this.client.mySelectedPoints.size() == 2){
			Iterator<IPoint> it = this.client.mySelectedPoints.iterator();
			line = this.client.myDiagram.connectPoints(it.next(),it.next());
			
			this.client.mySelectedPoints.clear();
			this.myCurrentPoint = null;
			this.client.removeNewLine();
			
		} else if (this.client.mySelectedPoints.size() == 1){
			IPoint point = this.client.myDiagram.addPoint(this.client.fromRealXToOur(x),
				this.client.fromRealYToOur(y));
			Iterator<IPoint> it = this.client.mySelectedPoints.iterator();
			line = this.client.myDiagram.connectPoints(it.next(), point);
			
			this.client.mySelectedPoints.clear();
			this.myCurrentPoint = null;
			this.client.removeNewLine();
		}else {
			this.client.mySelectedPoints.clear();
			this.myCurrentPoint = null;
			this.client.removeNewLine();
		}
		
		IPoint point = this.client.myDiagram.findPoint(this.client.fromRealXToOur(x),
				this.client.fromRealYToOur(y),
				this.client.myPointDiam);
		
		if (point != null){
			if (point.canConnect()){
				this.client.mySelectedPoints.clear();
				this.client.mySelectedPoints.add(point);
				this.myCurrentPoint = point;
			}
		}
		
		e.getComponent().repaint();
	}

	public void mouseDragged(MouseEvent e) {
		
		int x = e.getX();
		int y = e.getY();
		
		if (e.getComponent().getHeight() - this.client.myPointDiam < e.getY()){
			y = e.getComponent().getHeight() - this.client.myPointDiam;
		}
		
		if (e.getComponent().getWidth() - this.client.myPointDiam < e.getX()){
			x = e.getComponent().getWidth() - this.client.myPointDiam;
		}
		
		if (e.getY() < this.client.myPointDiam){
			y = this.client.myPointDiam;
		}
		
		if (e.getX() < this.client.myPointDiam){
			x = this.client.myPointDiam;
		}
		
		e.getComponent().repaint();
		if (this.myCurrentPoint != null){
			IPoint point = this.client.myDiagram.findPoint(this.client.fromRealXToOur(x),
					this.client.fromRealYToOur(y),
					this.client.myPointDiam);
			if ((point == null) || (!point.canConnect())){
				this.client.setNewLine(this.myCurrentPoint, x, y, false);
				this.client.mySelectedPoints.clear();
				this.client.mySelectedPoints.add(this.myCurrentPoint);
			} else {
				this.client.setNewLine(this.myCurrentPoint, 
						this.client.fromOurXToReal(point.getX()), this.client.fromOurYToReal(point.getY()), true);
				this.client.mySelectedPoints.clear();
				this.client.mySelectedPoints.add(this.myCurrentPoint);
				this.client.mySelectedPoints.add(point);
			}
		}
	}

	public void mouseMoved(MouseEvent e) {
		IPoint point = this.client.myDiagram.findPoint(this.client.fromRealXToOur(e.getX()),
				this.client.fromRealYToOur(e.getY()),
				this.client.myPointDiam);
		
		if (point != null){
			if (point.canConnect()){
				this.client.mySelectedPoints.clear();
				this.client.mySelectedPoints.add(point);
				this.myCurrentPoint = point;
			}
		} else {
			this.myCurrentPoint = null;
			this.client.mySelectedPoints.clear();
		}
		e.getComponent().repaint();
	}

	@Override
	public boolean isActionPossible() {
		return (this.client.myDiagram.getPoints().size() > 0) && !this.client.myDiagram.isComplete();
	}

	@Override
	public IEnum getActionCode() {
		return ActionEnum.ADD_LINE;
	}
	
}
