package org.beliaj.knots.painters.impl;

import java.awt.*;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import javax.swing.event.*;

import org.beliaj.knots.diagram.*;
import org.beliaj.knots.painters.*;

/**
 * ,   .
 * 
 * @author Julia
 *
 */
public class PaintDiagramClient implements IPainter {
	
	private static final double PREFFERED_WINDOW_SIZE = 0.5;
	private static final double MINIMUM_MODEL_SIZE = 100;
	//private static final double INTERSECTION_SIZE = 0.05;
	private static final int MY_HOME_SCREEN_HEIGHT = 1024;
	boolean myDir = false; 
	
	//public static final int ADD_POINT = 0;
	//public static final int ADD_LINE = 1;
	
	public void setCurrentAction(IEnum number){
		//System.out.println("Current Action " + number);
		this.myCurrentMouseListener = myMouseListeners.get(number);
	}
	
	/**
	 *  .
	 */
	Dimension myScreenSize = Toolkit.getDefaultToolkit().getScreenSize();
	
	Dimension myAxis;
	
	/**
	 *  .
	 */
	int myPointDiam = 10;
	
	/**
	 *   .
	 */
	private int myBorder = 50;
	
	Color myBgColor = Color.white;
	Color myMainColor = Color.black;
	Color mySelectionColor = new Color(127, 0, 85);
	Color myNewColor = new Color(99, 130, 191);
	
	double myIntSize = 50;
	double myLineLen = 10;
	
	private boolean hasNewLine = false;
	
	private int myNewX = 0;
	private int myNewY = 0;
	
	/**
	 * 
	 */
	IKnotDiagram myDiagram;
	
	HashSet<IPoint> mySelectedPoints = new HashSet<IPoint>();
	HashSet<ILine> mySelectedLines = new HashSet<ILine>();
	HashSet<IIntersection> mySelectedIntersections = new HashSet<IIntersection>();
	
	private Actor myCurrentMouseListener;
	private HashMap<ActionEnum,Actor> myMouseListeners = new HashMap<ActionEnum,Actor>();
	private IPoint myCurrentPoint;
	private boolean isOk;
	private List<ChangeListener> myHistoryListeners = new LinkedList<ChangeListener>();
	
	{
		this.myCurrentMouseListener = new PointActor(this);
		this.myMouseListeners.put(ActionEnum.ADD_POINT, this.myCurrentMouseListener);
		this.myMouseListeners.put(ActionEnum.ADD_LINE, new LineActor(this));
		this.myMouseListeners.put(ActionEnum.MOVE, new MoveActor(this));
		this.myMouseListeners.put(ActionEnum.DELETE, new DeleteActor(this));
		this.myMouseListeners.put(ActionEnum.SWAP, new IntrsActor(this));
		this.myMouseListeners.put(ActionEnum.SHANGE_DIRECTION, new DirActor(this));
	}
	
	
	public MouseInputListener getMouseListener(){
		return this.myCurrentMouseListener;
	}
	
	/**
	 * 
	 * @param d 
	 */
	public PaintDiagramClient(IKnotDiagram d, Dimension dimension){
		this.myDiagram = d;
		if (dimension == null){
			int max = Math.max(d.getMaxXCoordinate(), d.getMaxYCoordinate());
			int min = Math.min(d.getMinXCoordinate(), d.getMinYCoordinate());
			if (max - min < PaintDiagramClient.MINIMUM_MODEL_SIZE){
					//System.out.println(this.screenSize.getHeight());
				d.resize(this.myScreenSize.getHeight()/ (max - min)
					*PaintDiagramClient.PREFFERED_WINDOW_SIZE);
				//System.out.println(d);
				max = Math.max(d.getMaxXCoordinate(), d.getMaxYCoordinate());
				min = Math.min(d.getMinXCoordinate(), d.getMinYCoordinate());
			}
			
			//System.out.println(d);
		
			this.myPointDiam = (int)Math.round(this.myPointDiam*
				((double)this.myScreenSize.height)/PaintDiagramClient.MY_HOME_SCREEN_HEIGHT);
			this.myBorder = (int)Math.round(this.myBorder*
				((double)this.myScreenSize.height)/PaintDiagramClient.MY_HOME_SCREEN_HEIGHT);
			if (this.myDiagram.getPoints().size() == 0){
				this.myAxis = new Dimension(this.myBorder,
					this.myBorder);
			} else {
				this.myAxis = new Dimension(this.myBorder - min,
						this.myBorder + max);
			}
		} else {
			this.myPointDiam = (int)Math.round(this.myPointDiam*
					((double)this.myScreenSize.height)/PaintDiagramClient.MY_HOME_SCREEN_HEIGHT);
			this.myBorder = (int)Math.round(this.myBorder*
					((double)this.myScreenSize.height)/PaintDiagramClient.MY_HOME_SCREEN_HEIGHT);
			this.myAxis = new Dimension(this.myBorder - this.myDiagram.getMinXCoordinate(),
					this.myBorder + this.myDiagram.getMaxYCoordinate());
		}
	}

	/**
	 *  .
	 * 
	 * @param point
	 * @param g
	 */
	private void drawPoint(IPoint point, Graphics2D g){
		if (this.mySelectedPoints.contains(point)){
			g.setColor(this.mySelectionColor);
		}
		g.fillOval(this.fromOurXToReal(point.getX()) - this.myPointDiam/2,
				this.fromOurYToReal(point.getY()) - this.myPointDiam/2,
				this.myPointDiam,
				this.myPointDiam);
		g.drawOval(this.fromOurXToReal(point.getX()) - this.myPointDiam/2,
				this.fromOurYToReal(point.getY()) - this.myPointDiam/2,
				this.myPointDiam,
				this.myPointDiam);
		g.setColor(this.myMainColor);
	}
	
	double[] getIntersectionPosition(double x1, double y1, double  x2, double y2, double x3, double y3, double x4, double y4) {
	
		double d = ((x1-x2)*(y4-y3) - (x4-x3)*(y1-y2));
		double alpha = ((x4-x2)*(y4-y3) - (x4-x3)*(y4-y2))/d;
		
		double[] r = new double[2];
		r[0] = alpha*x1 + (1-alpha)*x2;
		r[1] = alpha*y1 + (1-alpha)*y2;
		
		return r;
	}
	
	void drawNewLine(int fx, int fy, int sx, int sy, Graphics2D g){
		if (!isOk){
			g.setColor(this.myNewColor);
		} else {
			g.setColor(this.mySelectionColor);
		}
		g.drawLine(this.fromOurXToReal(fx),
				this.fromOurYToReal(fy),
				sx,
				sy);
		
		g.fillOval(sx - this.myPointDiam/2,
				sy - this.myPointDiam/2,
				this.myPointDiam,
				this.myPointDiam);
		g.drawOval(sx - this.myPointDiam/2,
				sy - this.myPointDiam/2,
				this.myPointDiam,
				this.myPointDiam);
		
		g.setColor(this.myMainColor);
	}
	
	/**
	 *  .
	 * 
	 * @param line
	 * @param g
	 */
	private void drawLine(ILine line, Graphics2D g){
		if (this.mySelectedLines.contains(line)){
			g.setColor(this.mySelectionColor);
		}
		
		g.drawLine(this.fromOurXToReal(line.getFPoint().getX()),
				this.fromOurYToReal(line.getFPoint().getY()),
				this.fromOurXToReal(line.getSPoint().getX()),
				this.fromOurYToReal(line.getSPoint().getY()));
		
		List<IIntersection> list = line.getIntersections();
		
		if (this.myDir){
			//System.out.println(this.myDir);
			this.drawDirection(line, g);
		} else {
			//System.out.println("not " + this.myDir);
		}
		
		for (IIntersection i: list){
			if (line.equals(i.getDownLine())){
				
				ILine line1 = i.getUpLine();
				
				
				
				double ix = i.getPosition()[0];
				double iy = i.getPosition()[1];
				
				Color tmpColor = g.getColor();
				g.setColor(g.getBackground());
				g.fillOval(this.fromOurXToReal((int)Math.round(ix - this.myPointDiam/2)),
						this.fromOurYToReal((int)Math.round(iy + this.myPointDiam/2)),
						this.myPointDiam,
						this.myPointDiam);
				g.drawOval(this.fromOurXToReal((int)Math.round(ix - this.myPointDiam/2)),
						this.fromOurYToReal((int)Math.round(iy + this.myPointDiam/2)),
						this.myPointDiam,
						this.myPointDiam);
				
				double upfpointX = line1.getFPoint().getX();
				double upfpointY = line1.getFPoint().getY();
				double upspointX = line1.getSPoint().getX();
				double upspointY = line1.getSPoint().getY();
				
				double rf = Math.sqrt((upfpointX - ix)*(upfpointX - ix) + (upfpointY - iy)*(upfpointY - iy));
				double alpha = this.myPointDiam/(1.5*rf);
				double x1 = upfpointX*alpha + ix*(1-alpha);
				double y1 = upfpointY*alpha + iy*(1-alpha);
				
				double rs = Math.sqrt((upspointX - ix)*(upspointX - ix) + (upspointY - iy)*(upspointY - iy));
				alpha = this.myPointDiam/(1.5*rs);
				double x2 = upspointX*alpha + ix*(1-alpha);
				double y2 = upspointY*alpha + iy*(1-alpha);
				
				if (this.mySelectedLines.contains(line1)){
					g.setColor(this.mySelectionColor);
				} else {
					g.setColor(this.myMainColor);
				}
				
				g.drawLine(this.fromOurXToReal((int)x1),
						this.fromOurYToReal((int)y1),
						this.fromOurXToReal((int)x2),
						this.fromOurYToReal((int)y2));
				
				if (this.myDir){
					//System.out.println(this.myDir);
					this.drawDirection(line1, g);
				}
				
				g.setColor(tmpColor);
			}
		}
		
		g.setColor(this.myMainColor);
		
		if (this.mySelectedLines.contains(line)){
			g.setColor(this.mySelectionColor);
		}
		
		g.setColor(this.myMainColor);
	}
				
	private void drawDirection(ILine line, Graphics2D g) {
		double upfpointX = line.getFPoint().getX();
		double upfpointY = line.getFPoint().getY();
		double upspointX = line.getSPoint().getX();
		double upspointY = line.getSPoint().getY();
		double dist = Math.sqrt((upspointX - upfpointX)*(upspointX - upfpointX) 
				+ (upspointY - upfpointY)*(upspointY - upfpointY));
		int[] x = new int[3];
		int[] y = new int[3];
		double cos = 12/13.0;
		double sin = 1/13.0;
		
		double len2 = this.myPointDiam+5;
		double len = len2+30;
		x[0] = this.fromOurXToReal((int)Math.round(len*((upfpointX - upspointX)*cos - (upfpointY - upspointY)*sin)/(dist) + upspointX));
		x[1] = this.fromOurXToReal((int)Math.round(len*((upfpointX - upspointX)*cos + (upfpointY - upspointY)*sin)/(dist) + upspointX));
		
		y[0] = this.fromOurYToReal((int)Math.round(len*((upfpointX - upspointX)*sin + (upfpointY - upspointY)*cos)/(dist) + upspointY));
		y[1] = this.fromOurYToReal((int)Math.round(len*(-(upfpointX - upspointX)*sin + (upfpointY - upspointY)*cos)/(dist) + upspointY));
		
		x[2] = this.fromOurXToReal(line.getSPoint().getX() + (int)Math.round(len2*(upfpointX - upspointX)/dist));
		y[2] = this.fromOurYToReal(line.getSPoint().getY() + (int)Math.round(len2*(upfpointY - upspointY)/dist));
		
		//System.out.println(x[0] + " " + y[0] + " " +
		//		x[1] + " " + y[1] + " " +
		//		x[2] + " " + y[2]);
		
		g.drawPolygon(x, y, 3);
		g.fillPolygon(x, y, 3);
		
		
	}

	/**
	 * ,    .
	 */			
	public int getWidth(){
		if (this.myDiagram.getPoints().size() == 0){
			return Math.round((int)(PREFFERED_WINDOW_SIZE*myScreenSize.height));
		}
		int max = Math.max(this.myDiagram.getMaxXCoordinate(),
				this.myDiagram.getMaxYCoordinate());
		int min = Math.min(this.myDiagram.getMinXCoordinate(),
				this.myDiagram.getMinYCoordinate());
		return (Math.abs(max- min))+ 2*myBorder;
	}
		
	/**
	 * ,    .
	 */
	public int getHeight(){
		if (this.myDiagram.getPoints().size() == 0){
			return Math.round((int)(PREFFERED_WINDOW_SIZE*myScreenSize.height));
		}
		int max = Math.max(this.myDiagram.getMaxXCoordinate(),
				this.myDiagram.getMaxYCoordinate());
		int min = Math.min(this.myDiagram.getMinXCoordinate(),
				this.myDiagram.getMinYCoordinate());
		return (Math.abs(max- min))+2*myBorder;
	}
	
	/**
	 * .
	 */
	public void repaint(Graphics2D g){
		//if (this.myFirstTime){
		//	this.myGraphics = g;
		//	this.myFirstTime = false;
		//}
		g.setBackground(this.myBgColor);
		g.setColor(this.myBgColor);
		g.fillRect(0,0,g.getClip().getBounds().width,g.getClip().getBounds().height);
		g.setColor(this.myMainColor);
		g.setBackground(this.myBgColor);
		
		if (this.hasNewLine){
			this.drawNewLine(this.myCurrentPoint.getX(), this.myCurrentPoint.getY(), this.myNewX, this.myNewY, g);
		}
		
		List<ILine> lines = this.myDiagram.getLines();
		for (ILine line: lines){
			this.drawLine(line, g);
		}
		
		List<IPoint> points = this.myDiagram.getPoints();
		for (IPoint point: points){
			this.drawPoint(point, g);
		}
		
	}
	
	
	/**
	 *  .
	 * @param x
	 * @return
	 */
	int fromRealXToOur(int x){
		return x - this.myAxis.width;
	}
	
	/**
	 * 
	 * @param y
	 * @return
	 */
	int fromRealYToOur(int y){
		return this.myAxis.height - y;
	}
	
	/**
	 * 
	 * @param x
	 * @return
	 */
	int fromOurXToReal(int x){
		return x + this.myAxis.width;
	}
	
	/**
	 * 
	 * @param y
	 * @return
	 */
	int fromOurYToReal(int y){
		return this.myAxis.height - y;
	}

	public void increase(Double coeff) {
		this.myDiagram.resize(coeff);
	}

	void setNewLine(IPoint point, int x, int y, boolean isOk) {
		this.hasNewLine = true;
		this.isOk = isOk;
		this.myNewX = x;
		this.myNewY = y;
		this.myCurrentPoint = point;
	}
	
	void removeNewLine(){
		this.hasNewLine = false;
	}

	public IKnotDiagram getKnotDiagram() {
		return this.myDiagram;
	}

	public boolean isActionPossible(IEnum actionCode) {
		ActionEnum aec = (ActionEnum) actionCode;
		return this.myMouseListeners.get(aec).isActionPossible();
	}

	public IEnum getCurrentAction() {
		return this.myCurrentMouseListener.getActionCode();
	}

	public void viewDirections() {
		this.myDir = !this.myDir;
		
	}

	public boolean hasDirections() {
		return myDir;
	}

	public void addHistoryListener(ChangeListener listener) {
		this.myHistoryListeners .add(listener);
	}
	
	void sendHistoryEvent(ChangeEvent e){
		for (ChangeListener cl: this.myHistoryListeners){
			cl.stateChanged(e);
		}
	}

}
