package org.beliaj.knots.math;

import junit.framework.TestCase;

public class PolynomTestCase extends TestCase {
	
	/**
	 *   
	 */
	public void test1(){
		
		Polynom p2 = Polynom.createMonom(2);
		Polynom p1 = Polynom.createMonom(1);
		Polynom p0 = Polynom.createMonom(0);
		
		TestCase.assertEquals("test1_1",
				"1 * x^0 + 1 * x^1", p0.add(p1).toString());
		TestCase.assertEquals("test1_2",
				"1 * x^0 + 1 * x^1 + 1 * x^2", p0.add(p2).toString());
		TestCase.assertEquals("test1_3",
				"2 * x^0 + 2 * x^1 + 2 * x^2", p0.add(p0).toString());
		
		p0 = p1.add(p2);
		TestCase.assertEquals("test1_4",
				"1 * x^1 + 1 * x^2", p0.toString());
		p1 = Polynom.createMonom(1);
		p1.add(p2);
		TestCase.assertEquals("test1_5",
				"1 * x^1 + 1 * x^2", p1.toString());
		p1.add(Polynom.createMonom(10));
		TestCase.assertEquals("test1_6",
				"1 * x^1 + 1 * x^2 + 1 * x^10", p1.toString());
		p1.mult(0, -1);
		TestCase.assertEquals("test1_7",
				"- 1 * x^1 - 1 * x^2 - 1 * x^10", p1.toString());
		TestCase.assertEquals("test1_8",
				"- 1 * x^10", p1.add(p0).toString());
		
	}
	
	/**
	 *     
	 */ 
	public void test2(){
		
		Polynom p5 = Polynom.createMonom(5);
		Polynom p4 = Polynom.createMonom(4);
		Polynom p3 = Polynom.createMonom(3);
		Polynom p2 = Polynom.createMonom(2);
		Polynom p1 = Polynom.createMonom(1);
		Polynom p0 = Polynom.createMonom(0);
		
		TestCase.assertEquals("test2_1", "1 * x^0", p0.toString());
		TestCase.assertEquals("test2_2", "1 * x^1", p1.toString());
		TestCase.assertEquals("test2_3", "1 * x^2", p2.toString());
		TestCase.assertEquals("test2_4", "1 * x^3", p3.toString());
		TestCase.assertEquals("test2_5", "1 * x^4", p4.toString());
		TestCase.assertEquals("test2_6", "1 * x^5", p5.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_7",
				"2 * x^0", p0.toString());
		
		p0.add(p1);
		TestCase.assertEquals("test2_8",
				"2 * x^0 + 1 * x^1", p0.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_9",
				"4 * x^0 + 2 * x^1", p0.toString());
		
		p0.add(p2);
		TestCase.assertEquals("test2_10",
				"4 * x^0 + 2 * x^1 + 1 * x^2", p0.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_11",
				"8 * x^0 + 4 * x^1 + 2 * x^2", p0.toString());
		
		p0.add(p3);
		TestCase.assertEquals("test2_12",
				"8 * x^0 + 4 * x^1 + 2 * x^2 + 1 * x^3", p0.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_13",
				"16 * x^0 + 8 * x^1 + 4 * x^2 + 2 * x^3", p0.toString());
		
		p0.add(p4);
		TestCase.assertEquals("test2_14",
				"16 * x^0 + 8 * x^1 + 4 * x^2 + 2 * x^3 + 1 * x^4", p0.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_15",
				"32 * x^0 + 16 * x^1 + 8 * x^2 + 4 * x^3 + 2 * x^4", p0.toString());
		
		p0.add(p5);
		TestCase.assertEquals("test2_16",
				"32 * x^0 + 16 * x^1 + 8 * x^2 + 4 * x^3 + 2 * x^4 + 1 * x^5", p0.toString());
		
		p0.mult(0,2);
		TestCase.assertEquals("test2_17",
				"64 * x^0 + 32 * x^1 + 16 * x^2 + 8 * x^3 + 4 * x^4 + 2 * x^5", p0.toString());
		
		p0.mult(1,1);
		TestCase.assertEquals("test2_18",
				"64 * x^1 + 32 * x^2 + 16 * x^3 + 8 * x^4 + 4 * x^5 + 2 * x^6", p0.toString());
		
		p0.mult(-239,1);
		TestCase.assertEquals("test2_19",
				"64 * x^-238 + 32 * x^-237 + 16 * x^-236 + 8 * x^-235 + 4 * x^-234 + 2 * x^-233", p0.toString());
		
		p0.mult(100,10);
		TestCase.assertEquals("test2_20",
				"640 * x^-138 + 320 * x^-137 + 160 * x^-136 + 80 * x^-135 + 40 * x^-134 + 20 * x^-133", p0.toString());
		
		p0.mult(0,0);
		TestCase.assertEquals("test2_21",
				"", p0.toString());
		
		p0.mult(-200, 10);
		TestCase.assertEquals("test2_22",
				"", p0.toString());
		
	}
	
	/**
	 *  
	 *
	 */
	public void test3(){
		
		Polynom p5 = Polynom.createMonom(5);
		Polynom p4 = Polynom.createMonom(4);
		Polynom p1 = Polynom.createMonom(1);
		Polynom p0 = Polynom.createMonom(0);
		
		TestCase.assertEquals("test3_1", "1 * x^0", p0.toString());
		TestCase.assertEquals("test3_2", "1 * x^1", p1.toString());
		TestCase.assertEquals("test3_5", "1 * x^4", p4.toString());
		TestCase.assertEquals("test3_6", "1 * x^5", p5.toString());
		
		p0.mult(p4);
		TestCase.assertEquals("test3_7", "1 * x^4", p0.toString());
		
		p1.mult(p5);
		TestCase.assertEquals("test3_8", "1 * x^6", p1.toString());
		
		p0.add(p1);
		TestCase.assertEquals("test3_9", "1 * x^4 + 1 * x^6", p0.toString());
		
		p0.mult(p1);
		TestCase.assertEquals("test3_10", "1 * x^10 + 1 * x^12", p0.toString());
		
		p0.mult(p0);
		TestCase.assertEquals("test3_11", "1 * x^20 + 2 * x^22 + 1 * x^24", p0.toString());
		
		p1.add(p4);
		TestCase.assertEquals("test3_12", "1 * x^4 + 1 * x^6", p1.toString());
		
		p1.mult(p0);
		TestCase.assertEquals("test3_13",
				"1 * x^24 + 3 * x^26 + 3 * x^28 + 1 * x^30", p1.toString());
	}
	
	/**
	 *     
	 *
	 */
	public void test4(){
		
		Polynom p2 = Polynom.createMonom(2);
		Polynom p1 = Polynom.createMonom(1);
		Polynom p0 = Polynom.createMonom(0);
		
		p0.mult(0, 2);
		TestCase.assertEquals("test4_1",
				"2 * x^0", p0.toString());
		
		p0.degree(10);
		TestCase.assertEquals("test4_2",
				"1024 * x^0", p0.toString());
		
		p0.degree(0);
		TestCase.assertEquals("test4_3",
				"1 * x^0", p0.toString());
		
		p1.degree(50);
		TestCase.assertEquals("test4_4",
				"1 * x^50", p1.toString());
		
		p1.add(p2);
		TestCase.assertEquals("test4_5",
				"1 * x^2 + 1 * x^50", p1.toString());
		
		p1.degree(2);
		TestCase.assertEquals("test4_6",
				"1 * x^4 + 2 * x^52 + 1 * x^100", p1.toString());
		
		//p1.degree(2);
		//TestCase.assertEquals("test4_7.5",
		//		"1.0 * x^8 + 4.0 * x^56 + 6.0 * x^104 + 4.0 * x^152 + 1.0 * x^200", p1.toString());
		
		
		p1.degree(3);
		TestCase.assertEquals("test4_7",
				"1 * x^12 + 6 * x^60 + 15 * x^108 + 20 * x^156 + 15 * x^204 + 6 * x^252 + 1 * x^300", p1.toString());
		
	}
	
}
